'use strict';

exports.__esModule = true;
exports['default'] = createStore;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _rx = require('rx');

var _rx2 = _interopRequireDefault(_rx);

var _utilsIsPlainObject = require('./utils/isPlainObject');

var _utilsIsPlainObject2 = _interopRequireDefault(_utilsIsPlainObject);

function createStore(reducer, initState) {
  if (typeof reducer !== 'function') {
    throw new Error('Expected the reducer to be a function.');
  }

  var initAction = { type: '@@rx-redux/INIT_' + new Date().getTime() };
  var listeners = [];
  var dispatcher$ = new _rx2['default'].Subject();

  var currentReducer = reducer;
  var state = currentReducer(initState, initAction);

  function callListeners() {
    listeners.forEach(function (listener) {
      return listener();
    });
  }

  function dispatch(action) {
    dispatcher$.onNext(action);
    return action;
  }

  function subscribe(listener) {
    listeners.push(listener);

    return function unsubscribe() {
      var index = listeners.indexOf(listener);
      listeners.splice(index, 1);
    };
  }

  function replaceReducer(newReducer) {
    currentReducer = newReducer;
    dispatcher$.onNext(initAction);
  }

  function reduce(action) {
    if (!_utilsIsPlainObject2['default'](action)) {
      throw new Error('Actions must be plain objects.');
    }

    state = currentReducer(state, action);

    return state;
  }

  var state$ = dispatcher$.map(reduce).publish().refCount().startWith(state);

  // must call state$.subscribe() to start life cycle
  state$.subscribe(callListeners, function (err) {
    throw err;
  });

  return {
    state$: state$,
    dispatcher$: dispatcher$,
    getState: function getState() {
      return state;
    },
    dispatch: dispatch,
    subscribe: subscribe,
    getReducer: function getReducer() {
      return currentReducer;
    },
    replaceReducer: replaceReducer
  };
}

module.exports = exports['default'];